"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const nock = require("nock");
const Prowl_1 = require("../../../main/ts/prowl/Prowl");
describe('Prowl', () => {
    let p;
    let successAnswer = '<?xml version="1.0" encoding="UTF-8"?><prowl><success code="200" remaining="123" resetdate="456789"/></prowl>';
    let successRetrieveTokenAnswer = '<?xml version="1.0" encoding="UTF-8"?><prowl><success code="200" remaining="123" resetdate="456789"/><retrieve token="TOKEN" url="URL" /></prowl>';
    let successRetrieveApikeyAnswer = '<?xml version="1.0" encoding="UTF-8"?><prowl><success code="200" remaining="123" resetdate="456789"/><retrieve apikey="APIKEY" /></prowl>';
    let invalidApikeyAnswer = '<?xml version="1.0" encoding="UTF-8"?><prowl><error code="401">Invalid apikey</error></prowl>';
    before(() => {
        p = new Prowl_1.default();
    });
    describe('constructor', () => {
        it('should set the api endpoint', () => {
            let p = new Prowl_1.default('https://www.foo.bar');
            assert.equal(p.getApiEndpoint(), 'https://www.foo.bar');
        });
        it('should set the default api endpoint if not set', () => {
            let p = new Prowl_1.default();
            assert.equal(p.getApiEndpoint(), 'https://api.prowlapp.com/publicapi');
        });
    });
    describe('verify', () => {
        it('should build correct get request and parse answer', () => __awaiter(this, void 0, void 0, function* () {
            nock(p.getApiEndpoint())
                .get(/verify/)
                .reply((uri, requestBody) => {
                assert.equal(uri, '/publicapi/verify?apikey=valid');
                return [200, successAnswer];
            });
            let res = yield p.verify('valid');
            assert.equal(res.valid, true);
            assert.equal(res.remainingCalls, 123);
            assert.equal(res.remainingCallsResetDate, 456789);
        }));
        it('should return valid=false for an invalid apikey', () => __awaiter(this, void 0, void 0, function* () {
            nock(p.getApiEndpoint())
                .get(/verify/)
                .reply(401, invalidApikeyAnswer);
            let res = yield p.verify('invalid');
            assert.equal(res.valid, false);
        }));
    });
    describe('add', () => {
        it('should build correct post request and parse answer', () => __awaiter(this, void 0, void 0, function* () {
            nock(p.getApiEndpoint())
                .post(/add/)
                .reply((uri, requestBody) => {
                assert.equal(uri, '/publicapi/add');
                assert.equal(requestBody, 'apikey=valid&application=TestApp&event=Test&description=Test%20App%20Description');
                return [200, successAnswer];
            });
            let res = yield p.add({
                apiKey: 'valid',
                application: 'TestApp',
                event: 'Test',
                description: 'Test App Description'
            });
            assert.equal(res.remainingCalls, 123);
            assert.equal(res.remainingCallsResetDate, 456789);
        }));
    });
    describe('retrieveToken', () => {
        it('should build correct post request and parse answer', () => __awaiter(this, void 0, void 0, function* () {
            nock(p.getApiEndpoint())
                .get(/retrieve\/token/)
                .reply((uri, requestBody) => {
                assert.equal(uri, '/publicapi/retrieve/token?providerkey=provider_key');
                return [200, successRetrieveTokenAnswer];
            });
            let res = yield p.retrieveToken('provider_key');
            assert.equal(res.token, 'TOKEN');
            assert.equal(res.url, 'URL');
            assert.equal(res.remainingCalls, 123);
            assert.equal(res.remainingCallsResetDate, 456789);
        }));
    });
    describe('retrieveApikey', () => {
        it('should build correct post request and parse answer', () => __awaiter(this, void 0, void 0, function* () {
            nock(p.getApiEndpoint())
                .get(/retrieve\/apikey/)
                .reply((uri, requestBody) => {
                assert.equal(uri, '/publicapi/retrieve/apikey?providerkey=provider_key&token=token');
                return [200, successRetrieveApikeyAnswer];
            });
            let res = yield p.retrieveApikey('provider_key', 'token');
            assert.equal(res.apikey, 'APIKEY');
            assert.equal(res.remainingCalls, 123);
            assert.equal(res.remainingCallsResetDate, 456789);
        }));
    });
});
//# sourceMappingURL=ProwlTest.js.map